#import "XeeController.h"
#import "XeeView.h"
#import "XeeImage.h"
#import "XeeImageSaver.h"
#import "XeeSimpleLayout.h"



// XeeController

@implementation XeeController

-(id)init
{
	if(self=[super init])
	{
		currimage=nil;
    }
    return self;
}

-(void)dealloc
{
	[currimage setAnimating:NO];
	[currimage release];

	[window release];

	[super dealloc];
}

-(void)windowWillClose:(NSNotification *)notification
{
	[self release];
}

-(void)windowWillMiniaturize:(NSNotification *)notification
{
	[imageview copyGLtoQuartz];
	[window setOpaque:NO]; // required to make the Quartz underlay and the window shadow appear correctly
}

-(void)windowDidMiniaturize:(NSNotification *)notification
{
	[window setOpaque:YES];
}



-(void)setImage:(XeeImage *)image
{
	if(image!=currimage)
	{
		[currimage setUpdateAction:nil target:nil];
		[currimage setAnimatingDefault];

		[currimage autorelease];
		currimage=[image retain];

		[currimage setAnimating:NO];
		[currimage setUpdateAction:@selector(imageUpdated:) target:self];
	}

	if(currimage)
	{
		NSSize maxsize=[self maxViewSize];

		bool shrink=[[NSUserDefaults standardUserDefaults] boolForKey:@"shrinkToFit"];
		bool enlarge=[[NSUserDefaults standardUserDefaults] boolForKey:@"enlargeToFit"];

		float horiz_zoom=maxsize.width/(float)[currimage width];
		float vert_zoom=maxsize.height/(float)[currimage height];
		float min_zoom=horiz_zoom<vert_zoom?horiz_zoom:vert_zoom;

		zoom=1;
		if(shrink&&min_zoom<1) zoom=min_zoom;
		if(enlarge&&min_zoom>1) zoom=min_zoom;

		[currimage setFrame:0];
		[imageview setImage:currimage];

		NSSize newsize=NSMakeSize(zoom*(float)[currimage width],zoom*(float)[currimage height]);
		[imageview setImageSize:newsize];
		[self setViewSize:newsize];

		[imageview setFocus:NSMakePoint(0,0)];
	}
	else
	{
		[imageview setImage:nil];
		//[self setViewSize:NSMakeSize(384,1)];
	}

	[self updateStatusBar];
}

-(void)imageUpdated:(XeeImage *)image
{
	[imageview setNeedsDisplay:YES];
	[self updateStatusBar];
}

-(void)setZoom:(float)newzoom
{
	if(!currimage) return;

	NSSize newsize=NSMakeSize(newzoom*(float)[currimage width],newzoom*(float)[currimage height]);
	[imageview setImageSize:newsize];
	[self setViewSize:newsize];

	zoom=newzoom;

	[self updateStatusBar];
}

-(void)setFrame:(int)frame
{
	if(!currimage) return;
	if(frame==[currimage frame]) return;
	if([currimage frames]==0) return;

	if(frame<0) frame=0;
	if(frame>=[currimage frames]) frame=[currimage frame]-1;

	int oldwidth=[currimage width];
	int oldheight=[currimage height];

	[currimage setFrame:frame];

	if([currimage width]!=oldwidth||[currimage height]!=oldheight)
	{
		NSSize newsize=NSMakeSize(zoom*(float)[currimage width],zoom*(float)[currimage height]);
		[imageview setImageSize:newsize];
		[self setViewSize:newsize];

		[imageview setFocus:NSMakePoint(0,0)];
	}

	[imageview setNeedsDisplay:YES];
}

-(void)setViewSize:(NSSize)size
{
	NSRect screenframe=[self availableScreenSpace];
	NSRect windowframe=[window frame];
	NSSize viewsize=[imageview bounds].size;
	NSSize minsize=[self minViewSize];

	if(size.width<minsize.width) size.width=minsize.width;
	if(size.height<minsize.height) size.height=minsize.height;

	int borderwidth=windowframe.size.width-viewsize.width;
	int borderheight=windowframe.size.height-viewsize.height;
	int win_width=size.width+borderwidth;
	int win_height=size.height+borderheight;

	if(win_width>screenframe.size.width) win_width=screenframe.size.width;
	if(win_height>screenframe.size.height) win_height=screenframe.size.height;

	int win_x=windowframe.origin.x+windowframe.size.width/2-win_width/2;
	int win_y=windowframe.origin.y+windowframe.size.height/2-win_height/2;

	if(win_x<screenframe.origin.x) win_x=screenframe.origin.x;
	if(win_y<screenframe.origin.y) win_y=screenframe.origin.y;
	if(win_x+win_width>screenframe.origin.x+screenframe.size.width) win_x=screenframe.origin.x+screenframe.size.width-win_width;
	if(win_y+win_height>screenframe.origin.y+screenframe.size.height) win_y=screenframe.origin.y+screenframe.size.height-win_height;

//	int width=win_width-borderwidth;
//	int height=win_height-borderheight;
//	[imageview setFocus:focus];

	[window setFrame:NSMakeRect(win_x,win_y,win_width,win_height) display:YES];
	[window invalidateCursorRectsForView:imageview]; // just to make sure
}

-(NSSize)maxViewSize
{
	NSSize screensize=[self availableScreenSpace].size;
	NSSize windowsize=[window frame].size;
	NSSize viewsize=[imageview bounds].size;

	return NSMakeSize(screensize.width-windowsize.width+viewsize.width,screensize.height-windowsize.height+viewsize.height);
}

-(NSSize)minViewSize
{
	return NSMakeSize(256,128);
}

-(NSRect)availableScreenSpace
{
	return [[window screen] visibleFrame];
}

-(void)updateStatusBar
{
	if(currimage)
	{
		NSString *status=[NSString stringWithFormat:@"%@ %dx%d %@, %@, %d of %d  frames",
		[currimage descriptiveFilesize],
		[currimage width],[currimage height],[currimage format],[currimage depth],
		[currimage frame]+1,[currimage frames]];
		[statusbar setStringValue:status];
	}
	else
	{
		[statusbar setStringValue:@"No file loaded"];
	}
}




-(NSWindow *)window { return window; }



-(void)copy:(id)sender
{
	if(currimage)
	{
		NSData *tiff=[[currimage imageRep] TIFFRepresentation];
		NSPasteboard *pboard=[NSPasteboard generalPasteboard];

		[pboard declareTypes:[NSArray arrayWithObject:NSTIFFPboardType] owner:nil];
		[pboard setData:tiff forType:NSTIFFPboardType];
	}
}

-(void)saveImage:(id)sender
{
	if(currimage) [XeeSavePanel runSavePanelForImage:currimage window:window];
}



-(IBAction)frameSkipNext:(id)obj
{
	if(currimage)
	{
		int frame=[currimage frame];
		int frames=[currimage frames];
		[self setFrame:(frame+1)%frames];
	}
}

-(IBAction)frameSkipPrev:(id)obj
{
	if(currimage)
	{
		int frame=[currimage frame];
		int frames=[currimage frames];
		[self setFrame:(frame+frames-1)%frames];
	}
}

-(IBAction)animationToggle:(id)obj
{
	if(!currimage||![currimage animated]) return;

	[currimage setAnimating:![currimage animating]];
}


static float zoomlevels[]={0.03125,0.044,0.0625,0.09,0.125,0.18,0.25,0.35,0.5,0.70,1,1.5,2,3,4,6,8,11,16,23,32};
static int num_zoom=21;

-(IBAction)zoomIn:(id)sender
{
	int i;
	for(i=0;i<num_zoom-1;i++) if(zoomlevels[i]>zoom) break;

	[self setZoom:zoomlevels[i]];
}

-(IBAction)zoomOut:(id)sender
{
	int i;
	for(i=num_zoom-1;i>1;i--) if(zoomlevels[i]<zoom) break;

	[self setZoom:zoomlevels[i]];
}

-(IBAction)zoomActual:(id)sender
{
	[self setZoom:1];
}

-(IBAction)zoomFit:(id)sender
{
	NSSize maxsize=[self maxViewSize];

	float horiz_zoom=maxsize.width/(float)[currimage width];
	float vert_zoom=maxsize.height/(float)[currimage height];

	[self setZoom:horiz_zoom<vert_zoom?horiz_zoom:vert_zoom];
}

/*-(IBAction)fullScreen:(id)sender
{
	[[imageview openGLContext] setFullScreen];
}*/

@end
