#import <Cocoa/Cocoa.h>
#import <OpenGL/GL.h>
#import <OpenGL/GLu.h>
#import <QuickTime/QuickTimeComponents.h>



#define XeeImageStateFailed 0
#define XeeImageStateInit 1
#define XeeImageStateLoading 2
#define XeeImageStateLoaded 3



@interface XeeImage:NSObject
{
	NSString *filename;
	NSDictionary *attrs;

	NSString *format;
	int width,height;
	NSString *depth;
	BOOL transparent;
	NSColor *back;

	int state;
	BOOL stop;

	SEL updateselector;
	id updatetarget;
}

-(id)initWithFile:(NSString *)name firstBlock:(NSData *)block attributes:(NSDictionary *)attributes;

-(int)state;
-(void)setState:(int)newstate;
-(BOOL)loaded;
-(BOOL)failed;
-(BOOL)needsLoading;

-(void)load;
-(void)stopLoading;
-(BOOL)hasBeenStopped;

-(int)frames;
-(void)setFrame:(int)frame;
-(int)frame;

-(void)setUpdateAction:(SEL)selector target:(id)target;
-(void)triggerUpdateAction;
-(BOOL)animated;
-(void)setAnimating:(BOOL)animating;
-(void)setAnimatingDefault;
-(BOOL)animating;

-(void)draw:(NSPoint)point size:(NSSize)size bounds:(NSRect)bounds;

-(NSString *)filename;
-(int)filesize;
-(NSDate *)date;
-(NSString *)format;
-(int)width;
-(int)height;
-(NSString *)depth;
-(BOOL)transparent;
-(NSColor *)backgroundColor;

-(NSBitmapImageRep *)imageRep;

-(NSString *)descriptiveFilesize;
-(NSString *)descriptiveDate;

-(void)setFilename:(NSString *)name;
-(void)setFormat:(NSString *)fmt;
-(void)setDepth:(NSString *)d;
-(void)setTransparent:(BOOL)trans;
-(void)setBackgroundColor:(NSColor *)col;

-(void)setDepthIndexed:(int)colors;
-(void)setDepthGrey:(int)greys;
-(void)setDepthRGB:(int)bits;
-(void)setDepthRGBA:(int)bits;
-(void)setDepthGreyAlpha:(int)bits;

+(void)initialize;
+(XeeImage *)imageForFilename:(NSString *)filename;
+(void)registerImageClass:(Class)class;
+(NSArray *)fileTypes;

@end



@interface XeeMultiImage:XeeImage
{
	NSMutableArray *subimages;
	int currindex;
}

-(id)initWithFile:(NSString *)name firstBlock:(NSData *)block attributes:(NSDictionary *)attributes;
-(void)dealloc;

-(void)addSubImage:(XeeImage *)subimage;
-(void)addSubImages:(NSArray *)array;
-(void)subImageUpdate:(XeeImage *)subimage;
-(XeeImage *)currentSubImage;

-(int)frames;
-(void)setFrame:(int)frame;
-(int)frame;

-(void)draw:(NSPoint)point size:(NSSize)size bounds:(NSRect)bounds;

-(int)width;
-(int)height;
-(NSString *)depth;
-(BOOL)transparent;
-(NSColor *)backgroundColor;

-(NSBitmapImageRep *)imageRep;

@end
